package com.tzld.piaoquan.recommend.server.framework.recaller.provider;

import com.google.common.base.Function;
import com.google.common.cache.CacheLoader;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import com.tzld.piaoquan.recommend.server.framework.candidiate.Entry;
import com.tzld.piaoquan.recommend.server.framework.candidiate.Queue;
import com.tzld.piaoquan.recommend.server.framework.candidiate.QueueName;
import com.tzld.piaoquan.recommend.server.framework.utils.IndexUtils;
import com.tzld.piaoquan.recommend.server.framework.utils.RedisSmartClient;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nullable;


public class RedisBackedQueueWithoutMetaCacheLoader extends CacheLoader<QueueName, Pair<Long, Queue<String>>> {

    private static final Logger logger = LoggerFactory.getLogger(RedisBackedQueueWithoutMetaCacheLoader.class);
    private final RedisSmartClient client;

    public RedisBackedQueueWithoutMetaCacheLoader(RedisSmartClient client) {
        this.client = client;
    }

    @Override
    public Pair<Long, Queue<String>> load(QueueName name) throws Exception {
        long start = System.nanoTime();

        byte[] bytes = client.get(IndexUtils.convertKey(name, IndexUtils.queuePrefixV1));
        Index index = new Index(bytes);

        if (null == index || CollectionUtils.isEmpty(index.getIndexEntryList())) {
            logger.error("empty_index_fetch");
        }

        Queue<String> queue = new Queue<String>(name.toString());

        Iterable<Entry<String>> iterable = Iterables.transform(index.getIndexEntryList(), new Function<IndexEntry, Entry<String>>() {
            @Nullable
            @Override
            public Entry<String> apply(IndexEntry indexEntry) {
                Entry<String> entry = new Entry<String>("", indexEntry.getItemId());
                entry.addScore("ordering", indexEntry.getScore());
                return entry;
            }
        });

        queue.addAll(Lists.newArrayList(iterable));

        long end = System.nanoTime();
        return Pair.of(Long.MIN_VALUE, queue);
    }
}
